==> catboost-1.2.1.tar.gz: catboost-1.2.1/catboost_all_src/util/system/mktemp.cpp <==
#include "tempfile.h"

#include <util/folder/dirut.h>
#include <util/generic/yexception.h>

#include <cstring>

#ifdef _win32_
    #include "winint.h"
    #include <io.h>
#else
    #include <unistd.h>
#endif

extern "C" int mkstemps(char* path, int slen);

TString MakeTempName(const char* wrkDir, const char* prefix, const char* extension) {
#ifndef _win32_
    TString filePath;

    if (wrkDir && *wrkDir) {
        filePath += wrkDir;
    } else {
        filePath += GetSystemTempDir();
    }

    if (filePath.back() != '/') {
        filePath += '/';
    }

    if (prefix) {
        filePath += prefix;
    }

    filePath += "XXXXXX"; // mkstemps requirement

    size_t extensionPartLength = 0;
    if (extension && *extension) {
        if (extension[0] != '.') {
            filePath += '.';
            extensionPartLength += 1;
        }
        filePath += extension;
        extensionPartLength += strlen(extension);
    }

    int fd = mkstemps(const_cast<char*>(filePath.data()), extensionPartLength);
    if (fd >= 0) {
        close(fd);
        return filePath;
    }
#else
    char tmpDir[MAX_PATH + 1]; // +1 -- for terminating null character
    char filePath[MAX_PATH];
    const char* pDir = 0;

    if (wrkDir && *wrkDir) {
        pDir = wrkDir;
    } else if (GetTempPath(MAX_PATH + 1, tmpDir)) {
        pDir = tmpDir;
    }

    // it always takes up to 3 characters, no more
    if (GetTempFileName(pDir, (prefix) ? (prefix) : "yan", 0, filePath)) {
        return filePath;
    }
#endif

    ythrow TSystemError() << "can not create temp name(" << wrkDir << ", " << prefix << ", " << extension << ")";
}

==> catboost-1.2.6.tar.gz: catboost-1.2.6/catboost_all_src/util/system/mktemp.cpp <==
#include "tempfile.h"

#include <util/folder/dirut.h>
#include <util/generic/yexception.h>

#include <cstring>

#ifdef _win32_
    #include "winint.h"
    #include <io.h>
#else
    #include <unistd.h>
#endif

extern "C" int mkstemps(char* path, int slen);

TString MakeTempName(const char* wrkDir, const char* prefix, const char* extension) {
    TString filePath;

    if (wrkDir && *wrkDir) {
        filePath += wrkDir;
    } else {
        filePath += GetSystemTempDir();
    }

#ifdef _win32_
    // https://learn.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-gettempfilenamea?redirectedfrom=MSDN
    const unsigned int DirPathMaxLen = 247;
    if (filePath.length() <= DirPathMaxLen) {
        // it always takes up to 3 characters, no more
        char winFilePath[MAX_PATH];
        if (GetTempFileName(filePath.c_str(), (prefix) ? (prefix) : "yan", 0,
                            winFilePath)) {
            return winFilePath;
        }
    }
#endif // _win32_

    if (filePath.back() != '/') {
        filePath += '/';
    }

    if (prefix) {
        filePath += prefix;
    }

    filePath += "XXXXXX"; // mkstemps requirement

    size_t extensionPartLength = 0;
    if (extension && *extension) {
        if (extension[0] != '.') {
            filePath += '.';
            extensionPartLength += 1;
        }
        filePath += extension;
        extensionPartLength += strlen(extension);
    }

    int fd = mkstemps(const_cast<char*>(filePath.data()), extensionPartLength);
    if (fd >= 0) {
        close(fd);
        return filePath;
    }

    ythrow TSystemError() << "can not create temp name(" << wrkDir << ", " << prefix << ", " << extension << ")";
}
